# coding=utf-8

import os
import random
import numpy as np

import vot
import torch
import math
import networks.pysot.config_path as path
from networks.pysot.interface_siamrpn import SiamRPN
import sys
sys.path.append(os.path.join(os.path.dirname(__file__),'networks/pytracking_HJ'))
from networks.pytracking_HJ.pytracking.atom.ATOM import ATOM



random.seed(1)
np.random.seed(1)
torch.manual_seed(1)
torch.cuda.manual_seed(1)
torch.backends.cudnn.benchmark = False
torch.backends.cudnn.deterministic = True

os.environ["CUDA_VISIBLE_DEVICES"] = '0'
device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
class ARTCS_Tracker():
    def __init__(self, vot_flag=False):
        """Initialization
        """

        """Initialize"""
        self.vot_flag = vot_flag
        self.tracker_name = 'VOT19'

        """Initialize networks"""
        self.net_ATOM = ATOM()
        self.net_siam = SiamRPN(
            cfg_file=path.siamrpn_mb_cfg,
            snapshot=path.siamrpn_mb_snap
        )

    def init(self, init_img, region): # [x y w h]

        self.init_gt = region # [x y w h]


        self.net_siam.init(init_img, self.init_gt)
        self.net_ATOM.init(init_img, self.init_gt, use_iou_net=False)

        '''Copy some parameters from ATOM'''
        self.ATOM_search_scale = self.net_ATOM.atom.params.search_area_scale
        self.ATOM_image_size = self.net_ATOM.atom.params.max_image_sample_size



    def track(self, img):
        '''ATOM first'''
        rough_box, predict_iou, classify_score = self.net_ATOM.inference(img)# rough_box: (x1,y1,w,h)
        CF_loc = np.array([rough_box[0]+rough_box[2]/2, rough_box[1]+rough_box[3]/2]).astype(np.int) # Cx,Cy
        if classify_score > 0.1:
            self.net_siam.tracker.center_pos = CF_loc.copy()
        search_center = self.net_siam.tracker.center_pos.copy()
        """Siamese Network next"""
        '''SiamRPN++'''
        siam_box_rpn, siam_score,s_x = \
            self.net_siam.inference(img) # [x1 y1 x2 y2] score
        wh = self.net_siam.tracker.size
        search_area = (wh[0] * self.ATOM_search_scale)*(wh[1] * self.ATOM_search_scale)

        target_scale = math.sqrt(search_area / self.ATOM_image_size)
        self.net_ATOM.atom.target_scale = target_scale

        self.net_ATOM.atom.pos = torch.Tensor(
            [self.net_siam.tracker.center_pos[1], self.net_siam.tracker.center_pos[0]])
        predict_box = siam_box_rpn
        predict_score = siam_score


        if self.vot_flag: # [x y w h]
            return vot.Rectangle(
                predict_box[0],
                predict_box[1],
                predict_box[2],
                predict_box[3]
            )
        else:
            return predict_box, predict_score, s_x, search_center,classify_score