# coding=utf-8


import os
import numpy as np
import torch
from networks.pysot.pysot.core.config import cfg
from networks.pysot.pysot.models.model_builder import ModelBuilder
from networks.pysot.pysot.tracker.tracker_builder import build_tracker
from networks.pysot.pysot.utils.model_load import load_pretrain


class SiamRPN():
    def __init__(self, cfg_file, snapshot):
        # load config
        cfg.merge_from_file(cfg_file)

        # cur_dir = os.path.dirname(os.path.realpath(__file__))

        # create model
        self.model = ModelBuilder()
        # load model
        self.model = load_pretrain(self.model, snapshot).cuda().eval()
        # build tracker
        self.tracker = build_tracker(self.model)


    def init(self, img, box): # [x y w h]
        self.tracker.init(img, box)

    def inference(self, img):
        with torch.no_grad():
            outputs = self.tracker.track(img)

        box_rpn = outputs['bbox']
        score = outputs['best_score']

        # [x y w h] to [x1 y1 x2 y2]
        # box_rpn[2] = box_rpn[2] + box_rpn[0]
        # box_rpn[3] = box_rpn[3] + box_rpn[1]


        box_rpn = np.array(box_rpn).astype(float)


        # return box_mask, box_rpn, score, mask, find # [x1 y1 x2 y2] score
        return box_rpn, score, outputs['s_x']  # [x1 y1 x2 y2] score 搜索区域尺寸



