# -*- coding: utf-8 -*-
from pytracking.atom.ATOM import ATOM
import torch
import numpy as np
from time import time
from PIL import Image, ImageFont, ImageDraw
import cv2

import os
os.environ["CUDA_VISIBLE_DEVICES"] = '0'
device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
torch.backends.cudnn.benchmark = True


def overlap_ratio(bbox, boxes):
    """
    Compute overlap ratio.
    """
    x1 = bbox[0]
    y1 = bbox[1]
    x2 = bbox[0] + bbox[2] - 1
    y2 = bbox[1] + bbox[3] - 1

    """compute ground truth area"""
    gtArea = bbox[2] * bbox[3]

    boxes = np.reshape(boxes, [-1, 4])
    box_num = len(boxes)

    overlap_ratio = []
    for i in range(box_num):
        xA = max(x1, boxes[i, 0])
        yA = max(y1, boxes[i, 1])
        xB = min(x2, boxes[i, 0] + boxes[i, 2] - 1)
        yB = min(y2, boxes[i, 1] + boxes[i, 3] - 1)

        boxArea = boxes[i, 2] * boxes[i, 3]

        # compute the area of intersection rectangle
        interArea = (xB - xA + 1) * (yB - yA + 1)

        # no overlap
        if (x2 <= boxes[i, 0] or y2 <= boxes[i, 1]) or (
                (boxes[i, 0] + boxes[i, 2]) <= x1 or (boxes[i, 1] + boxes[i, 3]) <= y1):
            interArea = 0

        # compute IoU
        overlap_ratio.append(
            interArea / float(float(gtArea) + boxArea - interArea))

    return np.array(overlap_ratio)


def save_vot(
        vid_name, tracker_name='0', save_path='./',
        box_list=[], confidence_list=[], time_list=[], iou_list=[]
):
    """保存 vot 结果"""
    vid_path = os.path.join(save_path, tracker_name, 'longterm', '{:s}'.format(vid_name))
    iou_path = os.path.join(save_path, tracker_name, 'iou', '{:s}'.format(vid_name))
    if not os.path.exists(vid_path):
        os.makedirs(vid_path)
    if not os.path.exists(iou_path):
        os.makedirs(iou_path)

    # save box
    with open(os.path.join(vid_path, '{:s}_001.txt'.format(vid_name)), 'w') as f:
        f.write('1\n')
        f.writelines(box_list)
    # save confidence
    with open(os.path.join(vid_path, '{:s}_001_confidence.value'.format(vid_name)), 'w') as f:
        f.write('\n')
        f.writelines(confidence_list)
    # save time
    with open(os.path.join(vid_path, '{:s}_time.txt'.format(vid_name)), 'w') as f:
        f.write(time_list[0])
        f.writelines(time_list)

    # save iou
    with open(os.path.join(iou_path, '{:s}_001_confidence.value'.format(vid_name)), 'w') as f:
        f.write('1\n')
        f.writelines(iou_list)




save_path = './results'
lt_path = '/media/masterbin-iiau/WIN_SSD/LTB50/sequences'

tracker_name = 'ATOM_GT_Now'
path_iou_img = os.path.join(save_path, tracker_name, 'iou_img')


"""读取视频 name"""
vid_name = os.listdir(lt_path)
vid_name.remove('list.txt')
vid_name.sort()


"""Setup Model"""
atom = ATOM()



cv2.namedWindow(tracker_name, cv2.WND_PROP_FULLSCREEN)
for vid_id in [vid_name.index('yamaha')]: #range(1,35):
    '''如果tracker_name和video跟之前跑过的一样的话，那么就不继续运行了'''
    # if os.path.exists(os.path.join(save_path, tracker_name, 'longterm', '{:s}'.format(vid_name[vid_id]))):
    #     continue

    print('processing...', vid_name[vid_id])

    # with open(os.path.join(lt_path, vid_name[vid_id], 'groundtruth.txt')) as f:
    #     bbox_list = f.readlines()
    # gt = []
    # for line in bbox_list:
    #     tmp = line.split(',')
    #     gt.append(np.array([float(p) for p in tmp]))
    # gt = np.array(gt) # x y w h
    gt = np.loadtxt(os.path.join(lt_path, vid_name[vid_id], 'groundtruth.txt'),np.float32,delimiter=',')#(x,y,w,h)
    im_path = os.path.join(lt_path, vid_name[vid_id], 'color', '{:0>8d}.jpg'.format(0 + 1))
    init_img = cv2.imread(im_path)

    '''初始化ATOM，包括“状态估计模块”和“分类模块”'''
    '''我的推测： 
    状态估计模块： 初始化目标模板
    分类模块： 从第一帧选取训练样本，训练一个滤波器'''
    atom.init(init_img, gt[0])

    iou_list = []
    box_list = []
    confidence_list = []
    time_list = []
    for frame_i in range(1, len(gt)):

        if np.isnan(gt[frame_i][0]):
            IOU = 0
            predict_score = 0
        else:
            x, y, w, h = gt[frame_i].tolist()
            # target_pos = np.array([x + w / 2, y + h / 2])
            # target_sz = np.array([w, h])

            atom.atom.pos = torch.Tensor([round(y + h / 2), round(x + w / 2)])
            atom.atom.target_sz = torch.Tensor([round(h), round(w)])

            im_path = os.path.join(lt_path, vid_name[vid_id], 'color', '{:0>8d}.jpg'.format(frame_i + 1))
            cur_ori_img = cv2.imread(im_path)

            tic = time()
            predict_box, predict_score,classify_score = atom.inference(cur_ori_img)
            toc = time()

            """计算保存 IoU"""
            IOU = overlap_ratio(predict_box.reshape(-1), gt[frame_i].reshape(-1))[0]

            """显示结果"""
            # cur_ori_img[:, :, 1] = (mask > 0) * 255 + (mask == 0) * cur_ori_img[:, :, 1]
            im = Image.fromarray(cur_ori_img[:, :, ::-1])  # RGB
            draw = ImageDraw.Draw(im)
            draw.rectangle((gt[frame_i][0], gt[frame_i][1],
                            gt[frame_i][0] + gt[frame_i][2],
                            gt[frame_i][1] + gt[frame_i][3]), outline=(255, 0, 0), width=3)
            draw.rectangle((predict_box[0], predict_box[1],
                            predict_box[0] + predict_box[2],
                            predict_box[1] + predict_box[3]), outline=(0, 255, 0), width=3)
            # for i in range(nms_boxes.shape[0]):
            #     draw.rectangle((nms_boxes[i,0], nms_boxes[i,1],
            #                     nms_boxes[i,0] + nms_boxes[i,2],
            #                     nms_boxes[i,1] + nms_boxes[i,3]), outline=(255, 255, 0), width=1)

            im = im.resize((640*2, 360*2))
            draw = ImageDraw.Draw(im)
            font = ImageFont.truetype("./arial.ttf", 30)
            draw.text((5, 5), '# {:d}'.format(frame_i), fill=(255, 255, 0), font=font)
            draw.text((5, 40), 'IoU: {:.2f}'.format(IOU), fill=(255, 255, 0), font=font)
            # draw.text((5 ,80),'Pred_IoU:{:.2f}'.format(predict_score), fill=(0,255,255),font=font)
            draw.text((5, 80), 'classify_score:{:.2f}'.format(classify_score), fill=(255, 0, 255), font=font)
            draw.text((5, 680), 'ATOM', fill=(0, 255, 0), font=font)
            draw.text((95, 680), 'GT', fill=(255, 0, 0), font=font)

            res = np.array(im)[:, :, ::-1] # BGR
            cv2.imshow(tracker_name, res)
            cv2.waitKey(10)

            """根据 IoU 阈值保存较差结果"""
            if IOU < 0.6:
                path_iou_vid = os.path.join(path_iou_img, '{:s}'.format(vid_name[vid_id]))
                if not os.path.exists(path_iou_vid):
                    os.makedirs(path_iou_vid)
                tmp_path = os.path.join(
                    path_iou_vid,
                    '{:0>2d}_01_{:0>5d}_{:0>2d}.jpg'.format(vid_id, frame_i, int(IOU*100))
                )
                im.save(tmp_path)

        iou_list.append('{:.6f}\n'.format(IOU))
        box_list.append('{:.2f},{:.2f},{:.2f},{:.2f}\n'.format(predict_box[0], predict_box[1], predict_box[2], predict_box[3]))
        confidence_list.append('{:.6f}\n'.format(predict_score))
        time_list.append('{:.6f}\n'.format(toc-tic))

    """保存 vot 结果"""
    save_vot(
        vid_name[vid_id], tracker_name=tracker_name, save_path=save_path,
        box_list=box_list, confidence_list=confidence_list, time_list=time_list, iou_list=iou_list
     )


