# coding=utf-8
from pytracking.evaluation import Tracker
from pytracking.tracker.atom.atom import ATOM as atom_net
import torch
import torch.nn.functional as F
import torch.nn
import math
import numpy as np
from pytracking import dcf, fourier, TensorList, operation
from pytracking.features.preprocessing import numpy_to_torch


def numpy_to_torch(a: np.ndarray):
    return torch.from_numpy(a).float().permute(2, 0, 1).unsqueeze(0)

'''这个类是我们对原来ATOM模型的再次封装，目的是在tracking的过程中方便调用'''
class ATOM():
    def __init__(self):
        self.base_tracker = Tracker('atom', 'default_vot', None)
        self.atom = self.base_tracker.tracker_class(self.base_tracker.parameters)


    def init(self, img, gt_first, use_iou_net):
        self.atom.initialize(img, gt_first, use_iou_net)

    def inference_iou(self, img, box):

        """
        sample_pos 上一帧位置
        pos 新预测的位置
        self.pos [y x]
        self.target_sz [h w]
        """
        im = numpy_to_torch(img) # pytracking/features/deep.py 中提特征，自动转为 cuda.Tensor

        sz = torch.Tensor([box[3], box[2]])
        sz = torch.Tensor([110,50])

        pos = torch.Tensor([box[1], box[0]])
        sample_pos = torch.Tensor([box[1], box[0]])

        sample_scales = self.atom.target_scale * self.atom.params.scale_factors
        # forward 提特征
        _ = self.atom.extract_processed_sample(im, self.atom.pos, sample_scales, self.atom.img_sample_sz)

        box_center = (pos - sample_pos) / sample_scales[0] + (self.atom.iou_img_sample_sz - 1) / 2
        # box_center = 150 + (self.atom.iou_img_sample_sz - 1) / 2

        box_sz = sz / sample_scales[0]
        target_ul = box_center - (box_sz - 1) / 2
        test_box = torch.cat([target_ul.flip((0,)), box_sz.flip((0,))]) # [x y w h]

        iou_features = self.atom.get_iou_features()
        scale_ind = 0
        iou_features = TensorList([x[scale_ind:scale_ind+1,...] for x in iou_features])

        iou = self.atom.iou_predictor.predict_iou(self.atom.target_feat, iou_features, test_box.view(1,-1,4).cuda())/2+0.5

        # import cv2
        # import numpy as np
        # import matplotlib.pyplot as plt
        # tmp_im = np.zeros([288, 288, 3], dtype='uint8')
        # tmp_box = test_box.numpy()
        # tmp_box = tmp_box.astype(int)
        # tmp_box[:, 2:] += tmp_box[:, 0:2]
        # for i in range(tmp_box.shape[0]):
        #     cv2.rectangle(tmp_im, (tmp_box[i, 0], tmp_box[i, 1]), (tmp_box[i, 2], tmp_box[i, 3]), (0, 255, 0), 2)
        # plt.imshow(tmp_im)

        return iou

    def inference(self, img):
        state,predicted_iou,classify_score = self.atom.track(img) # [x y w h]
        box = np.array(state)
        return box, predicted_iou,classify_score