import numpy as np
from pytracking.evaluation.data import Sequence, BaseDataset, SequenceList


def VOTLTDataset():
    return VOTLTDatasetClass().get_sequence_list()


class VOTLTDatasetClass(BaseDataset):
    """VOT2018 lt dataset

    Publication:
        The sixth Visual Object Tracking VOT2018 challenge results.
        Matej Kristan, Ales Leonardis, Jiri Matas, Michael Felsberg, Roman Pfugfelder, Luka Cehovin Zajc, Tomas Vojir,
        Goutam Bhat, Alan Lukezic et al.
        ECCV, 2018
        https://prints.vicos.si/publications/365

    Download the dataset from http://www.votchallenge.net/vot2018/dataset.html"""
    def __init__(self):
        super().__init__()
        self.base_path = '/home/space/Documents/GitHub/vot-toolkit-channels/VOT19/sequences' #self.env_settings.vot_path
        self.sequence_list = self._get_sequence_list()

    def get_sequence_list(self):
        return SequenceList([self._construct_sequence(s) for s in self.sequence_list])

    def _construct_sequence(self, sequence_name):
        sequence_path = sequence_name
        nz = 8
        ext = 'jpg'
        start_frame = 1

        anno_path = '{}/{}/groundtruth.txt'.format(self.base_path, sequence_name)
        try:
            ground_truth_rect = np.loadtxt(str(anno_path), dtype=np.float64)
        except:
            ground_truth_rect = np.loadtxt(str(anno_path), delimiter=',', dtype=np.float64)

        end_frame = ground_truth_rect.shape[0]

        frames = ['{base_path}/{sequence_path}/color/{frame:0{nz}}.{ext}'.format(base_path=self.base_path,
                  sequence_path=sequence_path, frame=frame_num, nz=nz, ext=ext)
                  for frame_num in range(start_frame, end_frame+1)]

        # Convert gt
        if ground_truth_rect.shape[1] > 4:
            gt_x_all = ground_truth_rect[:, [0, 2, 4, 6]]
            gt_y_all = ground_truth_rect[:, [1, 3, 5, 7]]

            x1 = np.amin(gt_x_all, 1).reshape(-1,1)
            y1 = np.amin(gt_y_all, 1).reshape(-1,1)
            x2 = np.amax(gt_x_all, 1).reshape(-1,1)
            y2 = np.amax(gt_y_all, 1).reshape(-1,1)

            ground_truth_rect = np.concatenate((x1, y1, x2-x1, y2-y1), 1)

        return Sequence(sequence_name, frames, ground_truth_rect)

    def __len__(self):
        return len(self.sequence_list)

    def _get_sequence_list(self):
        # sequence_list= ['ballet',
        #                 'bicycle',
        #                 'bike1',
        #                 'bird1',
        #                 'car1',
        #                 'car3',
        #                 'car6',
        #                 'car8',
        #                 'car9',
        #                 'car16',
        #                 'carchase',
        #                 'cat1',
        #                 'cat2',
        #                 'dragon',
        #                 'following',
        #                 'freestyle',
        #                 'group1',
        #                 'group2',
        #                 'group3',
        #                 'liverRun',
        #                 'longboard',
        #                 'nissan',
        #                 'person2',
        #                 'person4',
        #                 'person5',
        #                 'person7',
        #                 'person14',
        #                 'person17',
        #                 'person19',
        #                 'person20',
        #                 'rollerman',
        #                 'skiing',
        #                 'tightrope',
        #                 'uav1',
        sequence_list= ['yamaha',
             'car16',
             'wingsuit',
             'carchase',
             'dog',
             'liverRun',
             'kitesurfing',
             'cat2',
             'sitcom',
             'group3',
             'bird1',
             'sup',
             'rollerman',
             'person7',
             'tightrope',
             'uav1',
             'car3',
             'person17',
             'car8',
             'longboard',
             'dragon',
             'deer',
             'person20',
             'freesbiedog',
             'helicopter',
             'cat1',
             'bicycle',
             'group2',
             'person5',
             'ballet',
             'volkswagen',
             'car9',
             'skiing',
             'horseride',
             'parachute',
             'person14',
             'nissan',
             'group1',
             'car1',
             'bike1',
             'bull',
             'boat',
             'person19',
             'car6',
             'following',
             'person2',
             'warmup',
             'freestyle',
             'person4',
             'f1']

        return sequence_list
